@@ -678,6 +678,9 @@ const gshort *gi_marshalling_tests_array_fixed_short_return (void);
 _GI_TEST_EXTERN
 void gi_marshalling_tests_array_fixed_int_in (const gint *ints);
 
+_GI_TEST_EXTERN
+void gi_marshalling_tests_array_fixed_caller_allocated_out (gint *ints);
+
 _GI_TEST_EXTERN
 void gi_marshalling_tests_array_fixed_short_in (const gshort *shorts);
 
@@ -689,6 +692,9 @@ void gi_marshalling_tests_array_fixed_out (gint **ints);
 _GI_TEST_EXTERN
 void gi_marshalling_tests_array_fixed_out_struct (GIMarshallingTestsSimpleStruct **structs);
 
+_GI_TEST_EXTERN
+void gi_marshalling_tests_array_fixed_caller_allocated_struct_out (GIMarshallingTestsSimpleStruct *structs);
+
 
 _GI_TEST_EXTERN
 void gi_marshalling_tests_array_fixed_inout (gint **ints);
@@ -1146,6 +1152,8 @@ void gi_marshalling_tests_gvalue_flat_array (guint         n_values,
 _GI_TEST_EXTERN
 GValue *gi_marshalling_tests_return_gvalue_flat_array (void);
 
+_GI_TEST_EXTERN
+GValue *gi_marshalling_tests_return_gvalue_zero_terminated_array (void);
 
 _GI_TEST_EXTERN
 GValue *gi_marshalling_tests_gvalue_round_trip (GValue *value);
@@ -1321,6 +1329,12 @@ typedef union {
     glong long_;
 } GIMarshallingTestsUnion;
 
+typedef union {
+    glong long_;
+    gsize size;
+    const gchar *str;
+} GIMarshallingTestsUnregisteredUnion;
+
 
 _GI_TEST_EXTERN
 GType gi_marshalling_tests_union_get_type (void) G_GNUC_CONST;
@@ -1337,6 +1351,79 @@ void gi_marshalling_tests_union_inv (GIMarshallingTestsUnion *union_);
 _GI_TEST_EXTERN
 void gi_marshalling_tests_union_method (GIMarshallingTestsUnion *union_);
 
+typedef enum
+{
+  GI_MARSHALLING_TESTS_STRUCTURED_UNION_TYPE_NONE,
+  GI_MARSHALLING_TESTS_STRUCTURED_UNION_TYPE_SIMPLE_STRUCT,
+  GI_MARSHALLING_TESTS_STRUCTURED_UNION_TYPE_NESTED_STRUCT,
+  GI_MARSHALLING_TESTS_STRUCTURED_UNION_TYPE_POINTER_STRUCT,
+  GI_MARSHALLING_TESTS_STRUCTURED_UNION_TYPE_BOXED_STRUCT,
+  GI_MARSHALLING_TESTS_STRUCTURED_UNION_TYPE_BOXED_STRUCT_PTR,
+  GI_MARSHALLING_TESTS_STRUCTURED_UNION_TYPE_SINGLE_UNION,
+} GIMarshallingTestsStructuredUnionType;
+
+typedef struct {
+    GIMarshallingTestsStructuredUnionType type;
+    GIMarshallingTestsSimpleStruct parent;
+} GIMarshallingTestsStructuredUnionSimpleStruct;
+
+typedef struct {
+    GIMarshallingTestsStructuredUnionType type;
+    GIMarshallingTestsNestedStruct parent;
+} GIMarshallingTestsStructuredUnionNestedStruct;
+
+typedef struct {
+    GIMarshallingTestsStructuredUnionType type;
+    GIMarshallingTestsPointerStruct parent;
+} GIMarshallingTestsStructuredUnionPointerStruct;
+
+typedef struct {
+    GIMarshallingTestsStructuredUnionType type;
+    GIMarshallingTestsBoxedStruct parent;
+} GIMarshallingTestsStructuredUnionBoxedStruct;
+
+typedef struct {
+    GIMarshallingTestsStructuredUnionType type;
+    GIMarshallingTestsBoxedStruct *parent;
+} GIMarshallingTestsStructuredUnionBoxedStructPtr;
+
+typedef struct {
+    GIMarshallingTestsStructuredUnionType type;
+    GIMarshallingTestsUnion union_;
+} GIMarshallingTestsStructuredUnionUnionStruct;
+
+typedef union {
+    GIMarshallingTestsStructuredUnionUnionStruct parent;
+} GIMarshallingTestsStructuredUnionSingleUnion;
+
+typedef union {
+    /*< private >*/
+    GIMarshallingTestsStructuredUnionType type;
+
+    GIMarshallingTestsStructuredUnionSimpleStruct simple_struct;
+    GIMarshallingTestsStructuredUnionNestedStruct nested_struct;
+    GIMarshallingTestsStructuredUnionPointerStruct pointer_struct;
+    GIMarshallingTestsStructuredUnionBoxedStruct boxed_struct;
+    GIMarshallingTestsStructuredUnionBoxedStructPtr boxed_struct_ptr;
+    GIMarshallingTestsStructuredUnionSingleUnion single_union;
+} GIMarshallingTestsStructuredUnion;
+
+_GI_TEST_EXTERN
+GType gi_marshalling_tests_structured_union_get_type (void) G_GNUC_CONST;
+
+_GI_TEST_EXTERN
+GIMarshallingTestsStructuredUnion *gi_marshalling_tests_structured_union_new (GIMarshallingTestsStructuredUnionType type);
+
+/* we want this to clash with a field value name */
+_GI_TEST_EXTERN
+GIMarshallingTestsStructuredUnionType gi_marshalling_tests_structured_union_type (GIMarshallingTestsStructuredUnion *structured_union);
+
+typedef union {
+    GIMarshallingTestsStructuredUnionType type;
+    GIMarshallingTestsStructuredUnionSimpleStruct simple_struct1;
+    GIMarshallingTestsStructuredUnionSimpleStruct simple_struct2;
+} GIMarshallingTestsStructuredUnionSingleType;
+
  /* Object */
 
 #define GI_MARSHALLING_TESTS_TYPE_OBJECT             (gi_marshalling_tests_object_get_type ())
@@ -2087,7 +2174,34 @@ GIMarshallingTestsSignalsObject *gi_marshalling_tests_signals_object_new (void);
 _GI_TEST_EXTERN
 void gi_marshalling_tests_signals_object_emit_boxed_gptrarray_utf8 (GIMarshallingTestsSignalsObject *object);
 
+_GI_TEST_EXTERN
+void gi_marshalling_tests_signals_object_emit_boxed_gptrarray_utf8_container (GIMarshallingTestsSignalsObject *object);
+
+_GI_TEST_EXTERN
+void gi_marshalling_tests_signals_object_emit_boxed_gptrarray_utf8_full (GIMarshallingTestsSignalsObject *object);
+
 _GI_TEST_EXTERN
 void gi_marshalling_tests_signals_object_emit_boxed_gptrarray_boxed_struct (GIMarshallingTestsSignalsObject *object);
 
+_GI_TEST_EXTERN
+void gi_marshalling_tests_signals_object_emit_boxed_gptrarray_boxed_struct_container (GIMarshallingTestsSignalsObject *object);
+
+_GI_TEST_EXTERN
+void gi_marshalling_tests_signals_object_emit_boxed_gptrarray_boxed_struct_full (GIMarshallingTestsSignalsObject *object);
+
+_GI_TEST_EXTERN
+void gi_marshalling_tests_signals_object_emit_hash_table_utf8_int (GIMarshallingTestsSignalsObject *object);
+
+_GI_TEST_EXTERN
+void gi_marshalling_tests_signals_object_emit_hash_table_utf8_int_container (GIMarshallingTestsSignalsObject *object);
+
+_GI_TEST_EXTERN
+void gi_marshalling_tests_signals_object_emit_hash_table_utf8_int_full (GIMarshallingTestsSignalsObject *object);
+
+_GI_TEST_EXTERN
+void gi_marshalling_tests_signals_object_emit_boxed_struct (GIMarshallingTestsSignalsObject *object);
+
+_GI_TEST_EXTERN
+void gi_marshalling_tests_signals_object_emit_boxed_struct_full (GIMarshallingTestsSignalsObject *object);
+
 #endif /* _GI_MARSHALLING_TESTS_H_ */
